﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Reflection;
using System.Resources;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Google.GData.Calendar;
using Google.GData.Client;
using Google.GData.Extensions;

namespace Thin_Calendar {
    public partial class ThinCalendar : Form {
        // We need this two unmanaged functions to set the desktop as parent
        [DllImport("user32.dll")]
        static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll")]
        static extern IntPtr SetParent(IntPtr child, IntPtr newParent);

        ResourceManager rm = new ResourceManager("Thin_Calendar.ThinCalendar", Assembly.GetExecutingAssembly());

        public ThinCalendar() {
            InitializeComponent();

            int year = DateTime.Now.Year;
            int month = DateTime.Now.Month;
            int numDays = DateTime.DaysInMonth(year, month);

            Thin_Calendar.Properties.Settings settings = new Thin_Calendar.Properties.Settings();

            List<string> titles = new List<string>();
            List<DateTime> dates = new List<DateTime>();

            if (settings.Email != "" && settings.Password != "") {
                // Authenticate to Google Calendar
                CalendarService service = new CalendarService("mundoGeek-thinCalendar-0.1");
                service.setUserCredentials(settings.Email, settings.Password);

                // Retrieve the events for this month
                EventQuery query = new EventQuery("http://www.google.com/calendar/feeds/default/private/full");

                query.StartTime = new DateTime(year, month, 1);
                query.EndTime = new DateTime(year, month, numDays);

                // Store the events in our data structures
                try {
                    EventFeed result = service.Query(query);
                    if (result != null)
                        foreach (EventEntry entry in result.Entries) {
                            foreach (When w in entry.Times) {
                                titles.Add(entry.Title.Text);
                                dates.Add(w.StartTime);
                            }
                        }
                } catch (InvalidCredentialsException) {
                    string messageError = rm.GetString("errorMessage");
                    string titleError = rm.GetString("errorTitle");

                    MessageBox.Show(messageError, titleError, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

            }
            string[] titlesArray = titles.ToArray();

            // Set the desktop as parent
            IntPtr hwndf = this.Handle;
            IntPtr hwndParent = FindWindow("ProgMan", null);
            SetParent(hwndf, hwndParent);

            // Create the cells for the days
            int screenWidth = SystemInformation.PrimaryMonitorSize.Width;
            int cellWidth = (screenWidth - 5 * (numDays + 1)) / numDays;
            int cellHeight = cellWidth;

            Font font = new Font("Arial", 10F, FontStyle.Bold, GraphicsUnit.Point, ((byte)(0)));
            Font fontItalic = new Font("Arial", 10F, FontStyle.Bold | FontStyle.Italic, GraphicsUnit.Point, ((byte)(0)));
            int minHeight = TextRenderer.MeasureText("L\n-\n1", font).Height;
            if (cellHeight < minHeight)
                cellHeight = minHeight;


            this.Size = new Size(screenWidth, cellHeight);
            this.Location = new Point(0, 0);

            int today = DateTime.Now.Day;


            Dictionary<string, string> weekDays = new Dictionary<string, string>();
            weekDays.Add("Monday", rm.GetString("monday"));
            weekDays.Add("Tuesday", rm.GetString("tuesday"));
            weekDays.Add("Wednesday", rm.GetString("wednesday"));
            weekDays.Add("Thursday", rm.GetString("thursday"));
            weekDays.Add("Friday", rm.GetString("friday"));
            weekDays.Add("Saturday", rm.GetString("saturday"));
            weekDays.Add("Sunday", rm.GetString("sunday"));

            // Add the labels with the days
            for (int i = 1; i <= numDays; i++) {
                Label lblDay = new Label();
                lblDay.Font = font;
                lblDay.BackColor = Color.FromArgb(46, 52, 54);
                lblDay.ForeColor = Color.FromArgb(238, 238, 236);
                lblDay.Location = new Point((i - 1) * cellWidth + i * 5, 0);
                lblDay.TextAlign = ContentAlignment.TopCenter;
                lblDay.Size = new Size(cellWidth, cellHeight);

                DateTime day = new DateTime(year, month, i);

                string dayWeek = weekDays[day.DayOfWeek.ToString()];
                lblDay.Text = dayWeek + "\n\n" + i.ToString();

                // Mark the current day
                if (i == today)
                    lblDay.Text = dayWeek + "\n\u25A0\n" + i.ToString();

                // Mark the days with events
                int index = 0;
                ToolTip tooltip = new ToolTip(new System.ComponentModel.Container());
                tooltip.ShowAlways = true;
                tooltip.IsBalloon = true;
                tooltip.ForeColor = Color.FromArgb(200, 200, 200);
                tooltip.BackColor = Color.FromArgb(46, 52, 54);

                foreach (DateTime date in dates) {
                    if (date == day) {
                        lblDay.BackColor = Color.FromArgb(164, 0, 0);
                        lblDay.ForeColor = Color.FromArgb(255, 255, 255);
                        string events = "";
                        if(tooltip.GetToolTip(lblDay) != "")
                            events = tooltip.GetToolTip(lblDay) + "\n " + titlesArray[index];
                        else
                            events = titlesArray[index];
                        tooltip.SetToolTip(lblDay, events);
                    }
                    index++;
                }


                this.Controls.Add(lblDay);
            }
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e) {
            Application.Exit();
            notifyIcon1.Dispose();
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e) {
            AboutBox about = new AboutBox();
            about.ShowDialog();
        }

        private void menuItemPreferences_Click(object sender, EventArgs e) {
            FormPreferences preferences = new FormPreferences();
            preferences.ShowDialog();
        }
    }
}